/*
    Program to demonstrate LinuxBot teleoperation

    Alan FT Winfield
    Intelligent Autonomous Systems Laboratory
    University of the West of England, Bristol

    Alan.Winfield@uwe.ac.uk

    Note:
    This program is - in effect - a TCP/IP client application, and therefore
    must be run together with a TCP/IP server, running in the LinuxBot,
    i.e. 'lserver.c'

      1. start 'lserver' on the LinuxBot, make sure it is 'listening...'
      2. press the 'Connect' button, enter the robot's IP address
      3. the lserver should then show '...connected'
      4. now use the mouse or arrow keys to drive the robot..

    Revision history:

    v1.0 August 2000
    v1.1 March 2001, no code changes, just tidied up the GUI.
*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include <stdio.h>
#include "Telebot.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TTelebotForm *TelebotForm;

//---------------------------------------------------------------------------
__fastcall TTelebotForm::TTelebotForm(TComponent* Owner)
        : TForm(Owner)
{
}
//---------------------------------------------------------------------------

void __fastcall TTelebotForm::FormCreate(TObject *Sender)
{
        bConnected = false;
}
//---------------------------------------------------------------------------

void __fastcall TTelebotForm::ConnectButtonClick(TObject *Sender)
{
	if(ClientSocket->Active)
    	        ClientSocket->Active = false;
  	if(InputQuery("LinuxBot IP Address..", "Address (IP):", Server))
    	        if(Server.Length() > 0)
                {
      		        ClientSocket->Address = Server;
                        ClientSocket->Open();
                        StatusBar->Panels->Items[0]->Text = "Trying";
                }
        TelebotForm->ActiveControl = NULL;
}
//---------------------------------------------------------------------------

void __fastcall TTelebotForm::SendButtonClick(TObject *Sender)
{
        if (bConnected)
                ClientSocket->Socket->SendText("04 ");
        TelebotForm->ActiveControl = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TTelebotForm::ClientSocketConnect(TObject *Sender,
      TCustomWinSocket *Socket)
{
        StatusBar->Panels->Items[0]->Text = "Connected to: "
                                            + Socket->RemoteAddress;
        bConnected = true;
}
//---------------------------------------------------------------------------

void __fastcall TTelebotForm::ClientSocketDisconnect(TObject *Sender,
      TCustomWinSocket *Socket)
{
        StatusBar->Panels->Items[0]->Text = "Disconnected";
        bConnected = false;
}
//---------------------------------------------------------------------------

void __fastcall TTelebotForm::ClientSocketError(TObject *Sender,
      TCustomWinSocket *Socket, TErrorEvent ErrorEvent, int &ErrorCode)
{
        StatusBar->Panels->Items[0]->Text = "Connect Error";
        bConnected = false;
        ErrorCode = 0;
}
//---------------------------------------------------------------------------

void __fastcall TTelebotForm::ClientSocketRead(TObject *Sender,
      TCustomWinSocket *Socket)
{
        Memo->Lines->Add(Socket->ReceiveText());
}
//---------------------------------------------------------------------------
void TTelebotForm::SendAllStop(void)
{
        if (bConnected)
                ClientSocket->Socket->SendText("03 ");
}

void TTelebotForm::SendMotors(int leftSpeed, int rightSpeed)
{
        char buf[32];

        sprintf( buf, "06 %06d %06d ", leftSpeed, rightSpeed );
        if (bConnected)
                ClientSocket->Socket->SendText( AnsiString(buf) );
}

void TTelebotForm::SendForward(void)
{
        int Speed = 100-FwdSpeed->Position;
        SendMotors( Speed, Speed );
}

void TTelebotForm::SendBack(void)
{
        int Speed = 100-BackSpeed->Position;
        SendMotors( -Speed, -Speed );
}

void TTelebotForm::SendLeftTurn(void)
{
        int Speed = 100-TurnSpeed->Position;
        SendMotors( -Speed, Speed );
}

void TTelebotForm::SendRightTurn(void)
{
        int Speed = 100-TurnSpeed->Position;
        SendMotors( Speed, -Speed );
}
//---------------------------------------------------------------------------

void __fastcall TTelebotForm::ExitButtonClick(TObject *Sender)
{
        if (bConnected)
        {
                ClientSocket->Socket->SendText("99 ");
                ClientSocket->Close();
        }
        Close();
}
//---------------------------------------------------------------------------

void __fastcall TTelebotForm::ForwardBtnMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
        SendForward();
}
//---------------------------------------------------------------------------

void __fastcall TTelebotForm::ForwardBtnMouseUp(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
        SendAllStop();
        TelebotForm->ActiveControl = NULL;
}
//---------------------------------------------------------------------------

void __fastcall TTelebotForm::BackBtnMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
        SendBack();
}
//---------------------------------------------------------------------------

void __fastcall TTelebotForm::BackBtnMouseUp(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
        SendAllStop();
        TelebotForm->ActiveControl = NULL;
}
//---------------------------------------------------------------------------

void __fastcall TTelebotForm::LeftBtnMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
        SendLeftTurn();
}
//---------------------------------------------------------------------------

void __fastcall TTelebotForm::LeftBtnMouseUp(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
        SendAllStop();
        TelebotForm->ActiveControl = NULL;
}
//---------------------------------------------------------------------------

void __fastcall TTelebotForm::RightBtnMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
        SendRightTurn();
}
//---------------------------------------------------------------------------

void __fastcall TTelebotForm::RightBtnMouseUp(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
        SendAllStop();
        TelebotForm->ActiveControl = NULL;
}
//---------------------------------------------------------------------------

void __fastcall TTelebotForm::FormKeyDown(TObject *Sender, WORD &Key,
      TShiftState Shift)
{
        switch( Key )
        {
                case 37:
                        LeftBtn->Enabled = false;
                        SendLeftTurn();
                        break;
                case 38:
                        ForwardBtn->Enabled = false;
                        SendForward();
                        break;
                case 39:
                        RightBtn->Enabled = false;
                        SendRightTurn();
                        break;
                case 40:
                        BackBtn->Enabled = false;
                        SendBack();
       }
}
//---------------------------------------------------------------------------

void __fastcall TTelebotForm::FormKeyUp(TObject *Sender, WORD &Key,
      TShiftState Shift)
{
        switch( Key )
        {
                case 37:
                        LeftBtn->Enabled = true;
                        SendAllStop();
                        break;
                case 38:
                        ForwardBtn->Enabled = true;
                        SendAllStop();
                        break;
                case 39:
                        RightBtn->Enabled = true;
                        SendAllStop();
                        break;
                case 40:
                        BackBtn->Enabled = true;
                        SendAllStop();
        }
}
//---------------------------------------------------------------------------

void __fastcall TTelebotForm::OLmodeButtonClick(TObject *Sender)
{
        if (bConnected)
                ClientSocket->Socket->SendText("90 1 ");
}
//---------------------------------------------------------------------------

void __fastcall TTelebotForm::PVmodeButtonClick(TObject *Sender)
{
        if (bConnected)
                ClientSocket->Socket->SendText("90 2 ");
}
//---------------------------------------------------------------------------

void __fastcall TTelebotForm::IVmodeButtonClick(TObject *Sender)
{
        if (bConnected)
                ClientSocket->Socket->SendText("90 3 ");
}
//---------------------------------------------------------------------------

